@@ -4,6 +4,8 @@ from __future__ import unicode_literals
 from __future__ import print_function
 
 from __future__ import absolute_import
+
+import json
 import logging
 import os
 
@@ -22,9 +24,11 @@ from threedigrid.admin.lines.prepare import (
 
 from threedigrid.admin.levees.prepare import PrepareLevees
 from threedigrid.admin.nodes.prepare import (
-    PrepareNodes, PrepareManholes, PrepareConnectionNodes)
+    PrepareNodes, PrepareManholes, PrepareConnectionNodes, PrepareCells)
 from threedigrid.admin.pumps.prepare import PreparePumps
 from threedigrid.admin.levees.models import Levees
+from threedigrid.orm.base.encoder import NumpyEncoder
+from threedigrid.orm.constants import EXPORT_METHOD_TO_EXTENSION_MAP
 
 logger = logging.getLogger(__name__)
 
@@ -141,6 +145,8 @@ class GridAdminH5Prepare(object):
         PrepareNodes.prepare_datasource(
             node_group, mapping1d, id_mapper, has_1d=has_1d)
 
+        PrepareCells.prepare_datasource(h5py_file, threedi_datasource)
+
         h5py_file['nodes'].attrs['prepared'] = 1
 
     @staticmethod
@@ -256,17 +262,37 @@ class GridAdminH5Prepare(object):
 
 class GridAdminH5Export(object):
     """
-    Exporter that saves shapefiles to the model's /preprocessed dir
+    Exporter that saves shapefiles/geopackage/geojson to destination folder
     """
 
-    def __init__(self, gridadmin_file, export_method='to_shape'):
+    def __init__(
+            self,
+            gridadmin_file,
+            export_method='to_shape',
+            target_epsg_code=None,
+            destination=None,
+            indent=None
+    ):
         """
 
         :param gridadmin_file: hdf5 file (full path)
+        :param export_method: (str) 'to_shape', 'to_gpkg' or 'to_geojson'
+        :param target_epsg_code: (str) epsg code
+        :param destination: path to folder the export folder, defaults to
+            the same folder as the gridadmin_file
         """
         self.ga = GridH5Admin(gridadmin_file)
-        self._dest = os.path.split(gridadmin_file)[0]
         self._export_method = export_method
+        self._extension = EXPORT_METHOD_TO_EXTENSION_MAP[export_method]
+        if not target_epsg_code:
+            self._epsg = self.ga.epsg_code
+        else:
+            self._epsg = target_epsg_code
+        if not destination:
+            self._dest = os.path.split(gridadmin_file)[0]
+        else:
+            self._dest = destination
+        self._indent = indent
 
     def export_all(self):
         """convenience function to run all exports"""
@@ -278,6 +304,60 @@ class GridAdminH5Export(object):
         self.export_2d_openwater_lines()
         self.export_2d_vertical_infiltration_lines()
 
+    def export_frontend(self):
+        self.export_breaches()
+        self.export_channels()
+        self.export_pipes()
+        self.export_weirs()
+        self.export_culverts()
+        self.export_orifices()
+        self.export_manholes()
+        self.export_nodes()
+        self.export_pumps()
+        self.export_levees()
+        self._export_2d_grid()
+        self._combine_all_frontend()
+
+    def _combine_all_frontend(self):
+        """Combines all exported geojsons in the destination folder into
+        one geojson named 'all'
+        """
+        if self._extension not in ('.json', '.geojson'):
+            logger.info("Can only combine exports with geojson")
+            return
+
+        export_types = {
+            'breaches', 'channels', 'pipes', 'weirs', 'culverts',
+            'orifices', 'manholes', constants.NODES, 'pumps', 'levees'
+        }
+
+        dest = os.path.join(self._dest, 'all' + self._extension)
+        with open(dest, 'w') as output_file:
+            features = []
+            for export_type in export_types:
+                intermediate_result = os.path.join(
+                    self._dest, export_type + self._extension
+                )
+                if not os.path.exists(intermediate_result):
+                    continue
+
+                with open(intermediate_result, 'r') as ir:
+                    geojson = json.load(ir)
+                    im_features = geojson['features']
+                    features += im_features
+
+            json.dump(
+                {"type": "FeatureCollection", "features": features},
+                output_file,
+                indent=self._indent,
+                cls=NumpyEncoder,
+            )
+
+    def export_1d_all(self):
+        self.export_nodes()
+        self.export_lines()
+        self.export_levees()
+
     def export_nodes(self):
         """
         writes shapefile of all 1D nodes
@@ -288,9 +368,13 @@ class GridAdminH5Export(object):
                 "skipping export nodes...".format(self.ga.model_name)
             )
             return
-        dest = os.path.join(self._dest, constants.NODES_SHP)
-        getattr(self.ga.nodes.subset(
-            constants.SUBSET_1D_ALL), self._export_method)(dest)
+        dest = os.path.join(self._dest, constants.NODES + self._extension)
+        getattr(
+            self.ga.nodes.subset(
+                constants.SUBSET_1D_ALL
+            ).reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
 
     def export_lines(self):
         """
@@ -302,9 +386,117 @@ class GridAdminH5Export(object):
                 "skipping export lines...".format(self.ga.model_name)
             )
             return
-        dest = os.path.join(self._dest, constants.LINES_SHP)
-        getattr(self.ga.lines.subset(
-            constants.SUBSET_1D_ALL), self._export_method)(dest)
+        dest = os.path.join(self._dest, constants.LINES + self._extension)
+        getattr(
+            self.ga.lines.subset(
+                constants.SUBSET_1D_ALL
+            ).reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
+
+    def export_breaches(self):
+        if not hasattr(self.ga, 'breaches'):
+            logger.info(
+                "[*] Model {} does not have 1D, "
+                "skipping export lines...".format(self.ga.model_name)
+            )
+            return
+        dest = os.path.join(self._dest, 'breaches' + self._extension)
+        getattr(
+            self.ga.breaches.reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
+
+    def export_channels(self):
+        if not hasattr(self.ga.lines, 'channels'):
+            logger.info(
+                "[*] Model {} does not have 1D, "
+                "skipping export lines...".format(self.ga.model_name)
+            )
+            return
+        dest = os.path.join(self._dest, 'channels' + self._extension)
+        getattr(
+            self.ga.lines.channels.reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
+
+    def export_pipes(self):
+        if not hasattr(self.ga.lines, 'pipes'):
+            logger.info(
+                "[*] Model {} does not have 1D, "
+                "skipping export lines...".format(self.ga.model_name)
+            )
+            return
+        dest = os.path.join(self._dest, 'pipes' + self._extension)
+        getattr(
+            self.ga.lines.pipes.reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
+
+    def export_weirs(self):
+        if not hasattr(self.ga.lines, 'weirs'):
+            logger.info(
+                "[*] Model {} does not have 1D, "
+                "skipping export lines...".format(self.ga.model_name)
+            )
+            return
+        dest = os.path.join(self._dest, 'weirs' + self._extension)
+        getattr(
+            self.ga.lines.weirs.reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
+
+    def export_culverts(self):
+        if not hasattr(self.ga.lines, 'culverts'):
+            logger.info(
+                "[*] Model {} does not have 1D, "
+                "skipping export lines...".format(self.ga.model_name)
+            )
+            return
+        dest = os.path.join(self._dest, 'culverts' + self._extension)
+        getattr(
+            self.ga.lines.culverts.reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
+
+    def export_orifices(self):
+        if not hasattr(self.ga.lines, 'orifices'):
+            logger.info(
+                "[*] Model {} does not have 1D, "
+                "skipping export lines...".format(self.ga.model_name)
+            )
+            return
+        dest = os.path.join(self._dest, 'orifices' + self._extension)
+        getattr(
+            self.ga.lines.orifices.reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
+
+    def export_manholes(self):
+        if not hasattr(self.ga.nodes, 'manholes'):
+            logger.info(
+                "[*] Model {} does not have 1D, "
+                "skipping export lines...".format(self.ga.model_name)
+            )
+            return
+        dest = os.path.join(self._dest, 'manholes' + self._extension)
+        getattr(
+            self.ga.nodes.manholes.reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
+
+    def export_pumps(self):
+        if not hasattr(self.ga, 'pumps'):
+            logger.info(
+                "[*] Model {} does not have 1D, "
+                "skipping export lines...".format(self.ga.model_name)
+            )
+            return
+        dest = os.path.join(self._dest, 'pumps' + self._extension)
+        getattr(
+            self.ga.pumps.reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
 
     def export_levees(self):
         """
@@ -316,8 +508,11 @@ class GridAdminH5Export(object):
                 "skipping export levees...".format(self.ga.model_name)
             )
             return
-        dest = os.path.join(self._dest, constants.LEVEES_SHP)
-        getattr(self.ga.levees, self._export_method)(dest)
+        dest = os.path.join(self._dest, constants.LEVEES + self._extension)
+        getattr(
+            self.ga.levees.reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
 
     def export_grid(self):
         """
@@ -335,9 +530,14 @@ class GridAdminH5Export(object):
                     self.ga.model_name)
             )
             return
-        dest = os.path.join(self._dest, constants.GROUNDWATER_LINES_SHP)
-        getattr(self.ga.lines.subset(
-            constants.SUBSET_2D_GROUNDWATER), self._export_method)(dest)
+        dest = os.path.join(
+            self._dest, constants.GROUNDWATER_LINES + self._extension
+        )
+        getattr(
+            self.ga.lines.subset(
+                constants.SUBSET_2D_GROUNDWATER).reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
 
     def export_2d_openwater_lines(self):
         if not self.ga.has_2d:
@@ -347,9 +547,14 @@ class GridAdminH5Export(object):
                     self.ga.model_name)
             )
             return
-        dest = os.path.join(self._dest, constants.OPEN_WATER_LINES_SHP)
-        getattr(self.ga.lines.subset(
-            constants.SUBSET_2D_OPEN_WATER), self._export_method)(dest)
+        dest = os.path.join(
+            self._dest, constants.OPEN_WATER_LINES + self._extension
+        )
+        getattr(
+            self.ga.lines.subset(
+                constants.SUBSET_2D_OPEN_WATER).reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
 
     def export_2d_vertical_infiltration_lines(self):
         if not self.ga.has_2d:
@@ -360,10 +565,15 @@ class GridAdminH5Export(object):
             )
             return
         dest = os.path.join(
-            self._dest, constants.VERTICAL_INFILTRATION_LINES_SHP)
+            self._dest,
+            constants.VERTICAL_INFILTRATION_LINES + self._extension
+        )
         getattr(
-            self.ga.lines.subset(constants.SUBSET_2D_VERTICAL_INFILTRATION),
-            self._export_method)(dest)
+            self.ga.lines.subset(
+                constants.SUBSET_2D_VERTICAL_INFILTRATION
+            ).reproject_to(self._epsg),
+            self._export_method
+        )(dest, indent=self._indent)
 
     def _export_groundwater_grid(self):
         if not self.ga.has_groundwater:
@@ -374,9 +584,14 @@ class GridAdminH5Export(object):
                 )
             )
             return
-        dest_gw = os.path.join(self._dest, constants.GROUNDWATER_SHP)
-        getattr(self.ga.cells.subset(
-            constants.SUBSET_2D_GROUNDWATER), self._export_method)(dest_gw)
+        dest_gw = os.path.join(
+            self._dest, constants.GROUNDWATER + self._extension
+        )
+        getattr(
+            self.ga.cells.subset(
+                constants.SUBSET_2D_GROUNDWATER).reproject_to(self._epsg),
+            self._export_method
+        )(dest_gw, indent=self._indent)
 
     def _export_2d_grid(self):
         if not self.ga.has_2d:
@@ -387,7 +602,11 @@ class GridAdminH5Export(object):
             )
             return
 
-        dest_ow = os.path.join(self._dest, constants.OPEN_WATER_SHP)
-
-        getattr(self.ga.cells.subset(
-            constants.SUBSET_2D_OPEN_WATER), self._export_method)(dest_ow)
+        dest_ow = os.path.join(
+            self._dest, constants.OPEN_WATER + self._extension
+        )
+        getattr(
+            self.ga.cells.subset(
+                constants.SUBSET_2D_OPEN_WATER).reproject_to(self._epsg),
+            self._export_method
+        )(dest_ow, indent=self._indent)
